import sys
from typing import Any, List, Optional

endl = object()


class Cout:
    """
    ```python
    from pycout import *

    cout << "Hello" << " World!" << endl
    ```
    """
    def __init__(self, line: str = "") -> None:
        self.line = line

    def __lshift__(self, other: Any) -> Optional["Cout"]:
        if other is endl:
            sys.stdout.write("".join(self.line))
            sys.stdout.write("\n")
            sys.stdout.flush()
            return None
        return Cout(self.line + str(other))

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, Cout) and self.line == other.line

    def __repr__(self) -> str:
        return ""

    def __str__(self) -> str:
        return self.line


cout = Cout()



import unittest

from pycout import *
from pycout.cout import Cout


class Test(unittest.TestCase):

    def test(self):
        # cast providing non-str does not break
        cout << 1000 << " hellos to the world!" << endl

        partial_cout = cout << "Hello" << " World!"

        self.assertEqual(partial_cout << 0, Cout("Hello World!0"))
        self.assertEqual(partial_cout << 1, Cout("Hello World!1"))

        self.assertIsNone(partial_cout << endl)
        self.assertEqual(partial_cout << 1, Cout("Hello World!1"))

        self.assertIsNone(partial_cout << endl)



from setuptools import setup  # type: ignore

setup(
    name="pycout",
    version="1.0.0-rc3",
    packages=["pycout"],
    url="http://github.com/EnzoBnl/pycout",
    license="Apache 2.",
    author="Enzo Bonnal",
    author_email="enzobonnal@gmail.com",
    description="""C++'s `cout << "Hello" << " World !" << endl;` in Python.""",
)
